define(function(require, exports, module) {
    var kity = require('../core/kity');
    var utils = require('../core/utils');

    var Minder = require('../core/minder');
    var MinderNode = require('../core/node');
    var Command = require('../core/command');
    var Module = require('../core/module');
    var Renderer = require('../core/render');

    var HighlightRenderer = kity.createClass('HighlightRenderer', {
        base: Renderer,

        create: function(node) {

            var highlight = new kity.Rect()
                .setId(utils.uuid('node_highlight'));

            this.bringToBack = true;

            return highlight;
        },

        update: function(highlight, node, box) {

            var shape = node.getStyle('shape');

            var paddingLeft = node.getStyle('padding-left'),
                paddingRight = node.getStyle('padding-right'),
                paddingTop = node.getStyle('padding-top'),
                paddingBottom = node.getStyle('padding-bottom');

            var outlineBox = {
                x: box.x - paddingLeft,
                y: box.y - paddingTop,
                width: box.width + paddingLeft + paddingRight,
                height: box.height + paddingTop + paddingBottom
            };


            var radius = node.getStyle('radius');
            // 天盘图圆形的情况
            if (shape && shape == 'circle') {
                var p = Math.pow;
                var r = Math.round;

                radius = r(Math.sqrt(p(outlineBox.width, 2) + p(outlineBox.height, 2)) / 2);

                outlineBox.x = box.cx - radius;
                outlineBox.y = box.cy - radius;
                outlineBox.width = 2 * radius;
                outlineBox.height = 2 * radius;
            }

            var prefix = node.isSelected() ? (node.getMinder().isFocused() ? 'selected-' : 'blur-selected-') : '';
            highlight
                .setPosition(outlineBox.x, outlineBox.y)
                .setSize(outlineBox.width, outlineBox.height)
                .setRadius(radius)
                .fill(node.getData('background') || node.getStyle(prefix + 'background') || node.getStyle('background'))
                .stroke(node.getStyle(prefix + 'stroke' || node.getStyle('stroke')),
                    node.getStyle(prefix + 'stroke-width'));

            return new kity.Box(outlineBox);
        }
    });

    Module.register('HighlightModule', function() {

        //找下 assignee下的所有子节点
        function getNodesByAssignee(minder, assignee) {
            var nodes = [];
            minder.getRoot().preTraverse(function(node) {
                var assigneeList = node.getData('assignee') || [];
                if (assigneeList.indexOf(assignee) !== -1 || nodes.indexOf(node.parent) !== -1) {
                    nodes.push(node);
                }
            });
            // 去除已选中节点，防止渲染高亮样式
            var selectedNodes = minder.getSelectedNodes();
            return nodes.filter(function (node) {
                return selectedNodes.indexOf(node) === -1;
            });
        }

        return {
            'commands': {
                /**
                 * @command highlightAssignee
                 * @state
                 *   0: 当前有选中的节点
                 *  -1: 当前没有选中的节点
                 */
                'highlightAssignee': kity.createClass('HighlightAssigneeCommand', {
                    base: Command,
                    execute: function(minder, assignee) {
                        this.setContentChanged(false);
                        if (!assignee) {
                            return;
                        }
                        var nodes = getNodesByAssignee(minder, assignee);
                        nodes.forEach(function (node) {
                            var outlineRenderer = node.getRenderer('OutlineRenderer');
                            if (outlineRenderer) {
                                var shape = outlineRenderer.getRenderShape();
                                if (shape) {
                                    shape.fill('#409eff');
                                }
                            }

                        });
                    },

                    queryState: function(minder, assignee) {
                        return 0;
                    }
                }),

                /**
                 * @command ClearStyle
                 * @description 移除选中节点的样式，包括字体、字号、粗体、斜体、背景色、字体色
                 * @state
                 *   0: 当前有选中的节点，并且至少有一个设置了至少一种样式
                 *  -1: 其它情况
                 */
                'clearHighlight': kity.createClass('ClearHighlightCommand', {
                    base: Command,
                    execute: function(minder, assignee) {
                        this.setContentChanged(false);
                        if (!assignee) {
                            return;
                        }
                        var nodes = getNodesByAssignee(minder, assignee);
                        minder.renderNodeBatch(nodes);
                    },

                    queryState: function(minder) {
                        return 0;
                    }
                })
            }
        };
    });
});
